#ifndef _FLE_UTILS_H
#define _FLE_UTILS_H

#include "EDE.h"

#define FLE_DIR		 (1<<1) //Directory
#define FLE_FILE 	 (1<<2) //Regular file
#define FLE_LINK 	 (1<<3) //Sym link (ignored on WIN32)
#define FLE_DEVICE (1<<4) //Logical disk (ignored on Linux)

class FLE_FileAttr
{
public:
	FLE_FileAttr() { size=0; free=used=0; flags=0; }
	bool parse(const char *filename);
	uint size; // size for files
	char time[128]; // time str
	uint64 free; //Free space
	uint64 used; //Used space
	int flags;
};

// Returns homepath in WIN32 and Linux
extern EDE_API char *fle_get_homedir();

extern EDE_API FLE_FileAttr *fle_file_attr(const char *name);

extern EDE_API bool fle_exists(const char *name);

// Get file array from given dir
extern EDE_API char **fle_get_files(char *path);

// Check if given name is dir
extern EDE_API bool fle_is_dir(const char* n);

// removes leading spaces
extern EDE_API char *fle_trimleft(char *string);

// removes trailing spaces
extern EDE_API char *fle_trimright(char *string);

// Removes trailing and leading spaces
#define fle_trim(string) fle_trimright(fle_trimleft(string))

extern EDE_API char *fle_tolower(char *string);
extern EDE_API char *fle_toupper(char *string);

extern EDE_API char *fle_strdup_printf(char *s1, ...);

extern EDE_API char** fle_split(const char *string, const char *delimiter, int max_tokens);

extern EDE_API int fle_start_child_process(char *cmd);

extern EDE_API void fle_freev (char **str_array);


class EDE_API FLE_Point
{
	int x_, y_;
public:
	FLE_Point() : x_(0), y_(0) {}
	FLE_Point(int x, int y) : x_(x), y_(y) {}
	int x() { return x_; }
	int y() { return y_; }
	void x(int x) { x_ = x; }
	void y(int y) { y_ = y; }
};

class EDE_API FLE_Size
{
	int width_, height_;
public:
	FLE_Size() : width_(0), height_(0) {}
	FLE_Size(int w, int h) : width_(w), height_(h) {}
	
	int width() { return width_; }
	int w() { return width_; }
	
	int h() { return height_; }	
	int height() { return height_; }
	
	void width(int w) { width_ = w; }
	void height(int h) { height_ = h; }
};

class EDE_API FLE_Rect
{
	int x_, y_, width_, height_;
public:
	FLE_Rect() : x_(0), y_(0), width_(0), height_(0) {}
	FLE_Rect(int x, int y, int w, int h) : x_(x), y_(y), width_(w), height_(h) {}
	inline bool posInRect(int xs, int ys)
	{ 	
		if( xs > x_  && xs < (x_ + width_) &&
	    ys > y_  && ys < (y_ + height_) )
		{
			return true;
		}
		return false;
 	}
	inline bool posInRect(FLE_Point p)
	{ 	
		if( p.x() > x_  && p.x() < (x_ + width_) &&
	    p.y() > y_  && p.y() < (y_ + height_) )
		{
			return true;
		}
		return false;
 	}
	int x() { return x_; }
	int y() { return y_; }
	
	int w() { return width(); }
	int width() { return width_; }
	
	int h() { return height(); }
	int height() { return height_; }	
	
	void x(int x) { x_ = x; }
	void y(int y) { y_ = y; }
	void w(int w) { width_ = w; }
	void h(int h) { height_ = h; }

	void set(int x, int y, int w, int h) { x_=x; y_=y; width_=w; height_=h; }
};

#endif
